package rds_test

var DescribeDBInstancesExample1 = `
<DescribeDBInstancesResponse xmlns="http://rds.amazonaws.com/doc/2013-09-09/">
  <DescribeDBInstancesResult>
    <DBInstances>
      <DBInstance>
        <ReadReplicaDBInstanceIdentifiers/>
        <LatestRestorableTime>2011-05-23T06:50:00Z</LatestRestorableTime>
        <Engine>mysql</Engine>
        <PendingModifiedValues/>
        <BackupRetentionPeriod>1</BackupRetentionPeriod>
        <MultiAZ>false</MultiAZ>
        <LicenseModel>general-public-license</LicenseModel>
        <DBInstanceStatus>available</DBInstanceStatus>
        <EngineVersion>5.1.50</EngineVersion>
        <Endpoint>
          <Port>3306</Port>
          <Address>simcoprod01.cu7u2t4uz396.us-east-1.rds.amazonaws.com</Address>
        </Endpoint>
        <DBInstanceIdentifier>simcoprod01</DBInstanceIdentifier>
        <DBName>simcoprod</DBName>
        <DBParameterGroups>
          <DBParameterGroup>
            <ParameterApplyStatus>in-sync</ParameterApplyStatus>
            <DBParameterGroupName>default.mysql5.1</DBParameterGroupName>
          </DBParameterGroup>
        </DBParameterGroups>
        <DBSecurityGroups>
          <DBSecurityGroup>
            <Status>active</Status>
            <DBSecurityGroupName>default</DBSecurityGroupName>
          </DBSecurityGroup>
        </DBSecurityGroups>
        <PreferredBackupWindow>00:00-00:30</PreferredBackupWindow>
        <AutoMinorVersionUpgrade>true</AutoMinorVersionUpgrade>
        <PreferredMaintenanceWindow>sat:07:30-sat:08:00</PreferredMaintenanceWindow>
        <AvailabilityZone>us-east-1a</AvailabilityZone>
        <InstanceCreateTime>2011-05-23T06:06:43.110Z</InstanceCreateTime>
        <AllocatedStorage>10</AllocatedStorage>
        <OptionGroupMemberships>
          <OptionGroupMembership>
            <OptionGroupName>default.mysql5.1</OptionGroupName>
            <Status>in-sync</Status>
          </OptionGroupMembership>
        </OptionGroupMemberships>
        <DBInstanceClass>db.m1.large</DBInstanceClass>
        <MasterUsername>master</MasterUsername>
        <PubliclyAccessible>false</PubliclyAccessible>
      </DBInstance>
    </DBInstances>
  </DescribeDBInstancesResult>
  <ResponseMetadata>
    <RequestId>9135fff3-8509-11e0-bd9b-a7b1ece36d51</RequestId>
  </ResponseMetadata>
</DescribeDBInstancesResponse>
`
